# -*- coding: utf-8 -*-
"""
@author: He Dekun
"""

import numpy as np

def func_Status(close, indicator, cost, sharpe_frequency, beta=1):
    """
    0. This function is for calculating backtest outputs.
    1. This function has 4 arguements and returns 1 list with 3 elements as output.
    2. Input arguments including:
        (1) close: array, close price in time series.
        (2) indicator: array, buy sell indicator in time series.
        (3) cost: float, cost percentage.
        (4) sharpe_frequency: int, number of observations in one year.
    3. Output 1 list with 3 elements:
        (1) Return: list.
        (2) Trade_number.
        (3) Annual_Sharpe.
    """
    
    # Initialize:
    Return = np.zeros(len(indicator))
    start = 0
    Trade_number = 0
    
    # Main:
    for i in range(2, len(indicator)):
        
        if indicator[i-1] == 0:
            Return[i] = Return[i-1]
        
        if indicator[i-1] == 1:
            if indicator[i-2] != 1:
                start = i-1
                Trade_number += 1
                Return[i-1] = Return[i-1] - cost
            Return_delta = (close[i] - close[i-1]) / close[start] * beta
            Return[i] = Return[i-1] + Return_delta
        
        if indicator[i-1] == -1:
            if indicator[i-2] != -1:
                start = i-1
                Trade_number += 1
                Return[i-1] = Return[i-1] - cost
            Return_delta = (close[i-1] - close[i]) / close[start] * beta
            Return[i] = Return[i-1] + Return_delta
    
    diff = Return[1:] - Return[:-1]
    Annual_Sharpe = np.mean(diff) / np.std(diff) * np.sqrt(sharpe_frequency)
    
    return Return, Trade_number, Annual_Sharpe


# -*- coding: utf-8 -*-
"""
@author: He Dekun
"""

import numpy as np
import pandas as pd

def func_BarCount(HA_Open, HA_Close, HA_PS, bar_pass_cut=2, ps_pass_cut=2):
    """
    0. This function is for counting indices of continuously up or down bar (skipping some small-ps bar).
    1. This function has 5 arguments and returns 1 DataFrame with 3 colunms as output.
    2. Input arguments including: 
        (1) HA_Open, HA_Close, HA_PS, all are DataFrame.
        (2) bar_pass_cut: int, optional, default is 2, number of small opposite bars we can skip.
        (3) ps_pass_cut: int, optional, default is 2, ps cut level of oppsite bar we can skip.
    3. Output are combined in 1 DataFrame with 3 columns, including:
        (1) bar_len: HA bar trend cycle length, up trend being positive number, while down trend being negative.
        (2) bar_idx: count bar index, always starts from 1 or -1, would use future data to update current count index. (Use this carefully)
        (3) bar_idx_live: cout bar index, may start from numbers larger than 1, would not use furture data.
    """
    
    # Initialize:
    PS = HA_PS.values
    bar_len = np.zeros_like(HA_Open)
    bar_idx = np.zeros_like(HA_Open)
    bar_idx_live = np.zeros_like(HA_Open)
    
    idx_first = np.where(PS != 0)[0][0]
    bar_stats = np.zeros((len(HA_Open),3))
    bar_pass = 0
    idx = 0
    
    # Main:
    for i in range(idx_first, len(HA_Open)):
        bar_temp = np.sign(HA_Close.values[i] - HA_Open.values[i])
        # Up trend:
        if bar_temp == 1:
            if idx >= 0:
                idx += 1
                bar_stats[i, 0] = idx
                bar_idx_live[i] = idx
                bar_pass = 0
            else:
                if (PS[i] < ps_pass_cut) and (bar_pass < bar_pass_cut):
                    idx -= 1
                    bar_stats[i, 0] = idx
                    bar_idx_live[i] = idx
                    bar_pass += 1
                    if PS[i] == 0: bar_pass -= 1
                else:
                    bar_stats[i-bar_pass, 1] = idx + bar_pass
                    idx = bar_pass + 1
                    bar_stats[i, 0] = idx
                    bar_idx_live[i] = idx
                    for k in range(1, bar_pass+1): bar_stats[i-k, 0] = bar_pass + 1 - k
                    bar_pass = 0
        # Up trend:
        if bar_temp == -1:
            if idx <= 0:
                idx -= 1
                bar_stats[i, 0] = idx
                bar_idx_live[i] = idx
                bar_pass = 0
            else:
                if (PS[i] > -ps_pass_cut) and (bar_pass < bar_pass_cut):
                    idx += 1
                    bar_stats[i, 0] = idx
                    bar_idx_live[i] = idx
                    bar_pass += 1
                    if PS[i] == 0: bar_pass -= 1
                else:
                    bar_stats[i-bar_pass, 1] = idx - bar_pass
                    idx = -bar_pass - 1
                    bar_stats[i, 0] = idx
                    bar_idx_live[i] = idx
                    for k in range(1, bar_pass+1): bar_stats[i-k, 0] = -(bar_pass + 1 - k)
                    bar_pass = 0
    
    bar_stats[:, 2] = abs(bar_stats[:, 1])
    bar_idx[:] = bar_stats[:, 0]
    bar_len[:] = bar_stats[:, 2]
    
    bar_len_df      = pd.DataFrame(bar_len, columns=['bar_len'])
    bar_idx_df      = pd.DataFrame(bar_idx, columns=['bar_idx'])
    bar_idx_live_df = pd.DataFrame(bar_idx_live, columns=['bar_idx_live'])
    output_df = pd.concat([bar_len_df, bar_idx_df, bar_idx_live_df], axis=1)
    
    return output_df


# -*- coding: utf-8 -*-
"""
@author: He Dekun
"""

import pandas as pd
from copy import deepcopy

def func_BarHA(Open, High, Low, Close):
    """
    0. This function is for transferring normal Bar to Heikin Ashi Bar.
    1. This function has 4 arguements and returns 1 Dataframe with 4 columns as output.
    2. Input arguments including: Open, High, Low, Close. All should be Dataframe.
    3. Output 1 Dataframe with 4 columns: HA_Open, HA_High, HA_Low, HA_Close.
    """
    
    # Initialize:
    HA_Open  = deepcopy(Open).values
    HA_High  = deepcopy(High).values
    HA_Low   = deepcopy(Low).values
    HA_Close = deepcopy(Close).values
    
    # Main:
    for idx in range(1, len(HA_Close)):
        HA_Close[idx] = (Open.values[idx] + High.values[idx] + Low.values[idx] + Close.values[idx]) / 4
        HA_Open [idx] = (HA_Open[idx-1] + HA_Close[idx-1]) / 2
        HA_High [idx] = max(HA_Open[idx], HA_Close[idx], High.values[idx]) 
        HA_Low  [idx] = min(HA_Open[idx], HA_Close[idx], Low.values[idx]) 
    
    HA_Open  = pd.DataFrame(HA_Open,  columns=['HA_Open'])
    HA_High  = pd.DataFrame(HA_High,  columns=['HA_High'])
    HA_Low   = pd.DataFrame(HA_Low,   columns=['HA_Low' ])
    HA_Close = pd.DataFrame(HA_Close, columns=['HA_Close']) 
    
    HA = pd.concat([HA_Open, HA_High, HA_Low, HA_Close], axis=1)
    
    return HA


# -*- coding: utf-8 -*-
"""
@author: He Dekun
"""

import pandas as pd

def func_1min2day(data_min, cut_time_gap=5/24):
    """
    0. This function is for transferring a 1min bar to 1day bar, must has non-trading time over 5 hours in one day.
    1. This function has 1 argument and returns 1 Dataframe with 5 columns as output.
    2. Input argument:
        data_min: must be as Dataframe format, with 5 columns including 'time', 'high', 'low', 'close', 'open'.
    3. Output is a Dataframe with 5 columns including 'time_d', 'high_d', 'low_d', 'close_d', 'open_d'.
    
    NOTE:
        No need to change the input Dataframe's column names.
    """
    
    # Initialize:
    cut_time_gap = cut_time_gap   # for gapping time more than 5 hours
    df_input = pd.DataFrame()
    df_output = pd.DataFrame()
    
    for i in ['time', 'high', 'low', 'close', 'open']:
        for c in data_min.columns:
            if i in c:
                df_input[i] = data_min[c]
    
    time_d, high_d, low_d, close_d, open_d = [], [], [], [], []
    last_close = -1
    num_1min = len(data_min)
    
    # Main：
    for i in range(0, num_1min-1):
        if (df_input.time[i+1] - df_input.time[i]) > cut_time_gap:
            time_d.append(df_input.time[i])
            open_d.append(df_input.open[last_close+1])
            close_d.append(df_input.close[i])
            high_d.append(max(df_input.high[last_close+1:i]))
            low_d.append(min(df_input.low[last_close+1:i]))
            last_close = i
            
        if (i==num_1min-2) and ((df_input.time[i+1]-df_input.time[i]) <= cut_time_gap):
            time_d.append(df_input.time[i+1])
            open_d.append(df_input.open[last_close+1])
            close_d.append(df_input.close[i+1])
            high_d.append(max(df_input.high[last_close+1:i+1]))
            low_d.append(min(df_input.low[last_close+1:i+1]))
    
    data_dict = {'time_d':time_d, 'high_d':high_d, 'low_d':low_d, 'close_d':close_d, 'open_d':open_d}
    df_output = pd.DataFrame(data_dict)
    df_output = df_output[['time_d', 'high_d', 'low_d', 'close_d', 'open_d']]
    
    return df_output


# -*- coding: utf-8 -*-
"""
@author: He Dekun
"""

from . sampledata import data
from . process import func_BarHA, func_BarPS, func_BarCount, func_ChangeGranularity
from . backtest import func_Status, func_Momentum
from . plotting import func_plot_type1


class SampleData(object):
    
    def __init__(self):
        self.name = 'SampleData'
        
    def get_SampleData(self):
        return data.data()


class Process(object):
    
    def __init__(self):
        self.name = 'Process'
        
    def chg_Granularity(self, data_1min, granularity):
        return func_ChangeGranularity.func_ChangeGranularity(data_1min, granularity)
    
    def get_BarHA(self, Open, High, Low, Close):
        return func_BarHA.func_BarHA(Open, High, Low, Close)
    
    def get_BarPS(self, HA_Open, HA_Close, HA_PS_Lookback, PS_pct_level=[0.35, 0.5, 0.95, 0.97], combine=False):
        return func_BarPS.func_BarPS(HA_Open, HA_Close, HA_PS_Lookback, PS_pct_level, combine)
    
    def get_BarCT(self, HA_Open, HA_Close, HA_PS, bar_pass_cut=2, ps_pass_cut=2):
        return func_BarCount.func_BarCount(HA_Open, HA_Close, HA_PS, bar_pass_cut, ps_pass_cut)


class BackTest(object):
    
    def __init__(self):
        self.name = 'BackTest'
        
    def backtest_Momentum(self, data_PS, data_CT):
        return func_Momentum.func_Momentum(data_PS, data_CT)

    def get_PnL(self, close, indicator, cost, sharpe_frequency, beta=1):
        return func_Status.func_Status(close, indicator, cost, sharpe_frequency, beta)
    
    
class Plotting(object):
    
    def __init__(self):
        self.name = 'Plotting'
        
    def plot_type1(self, Open, High, Low, Close, indicator, result):
        return func_plot_type1.func_plot_type1(Open, High, Low, Close, indicator, result)
    